@@ -95,7 +95,7 @@ class DoxygenToRstRendererFactory(object):
             data_object
             ):
 
-        if not self.filter_.allow(parent_data_object, data_object):
+        if not self.filter_.allow([data_object, parent_data_object]):
             return NullRenderer()
 
         child_renderer_factory = self.renderer_factory_creator.create_child_factory(
@@ -106,7 +106,7 @@ class DoxygenToRstRendererFactory(object):
 
         try:
             node_type = data_object.node_type
-        except AttributeError, e:
+        except AttributeError as e:
 
             # Horrible hack to silence errors on filtering unicode objects
             # until we fix the parsing
@@ -142,9 +142,9 @@ class DoxygenToRstRendererFactory(object):
             elif data_object.type_ == "subscript":
                 creator = self.node_factory.subscript
             elif data_object.type_ == "center":
-                print "Warning: does not currently handle 'center' text display"
+                print("Warning: does not currently handle 'center' text display")
             elif data_object.type_ == "small":
-                print "Warning: does not currently handle 'small' text display"
+                print("Warning: does not currently handle 'small' text display")
 
             return Renderer(
                     creator,
@@ -276,11 +276,13 @@ class DoxygenToRstRendererFactoryCreator(object):
             "verbatim" : compoundrenderer.VerbatimTypeSubRenderer,
             "mixedcontainer" : compoundrenderer.MixedContainerRenderer,
             "unicode" : UnicodeRenderer,
+            "doclist": compoundrenderer.DocListTypeSubRenderer,
+            "doclistitem": compoundrenderer.DocListItemTypeSubRenderer,
             }
 
         try:
             node_type = data_object.node_type
-        except AttributeError, e:
+        except AttributeError as e:
 
             # Horrible hack to silence errors on filtering unicode objects
             # until we fix the parsing
@@ -323,7 +325,7 @@ class DoxygenToRstRendererFactoryCreator(object):
 
         try:
             node_type = data_object.node_type
-        except AttributeError, e:
+        except AttributeError as e:
 
             # Horrible hack to silence errors on filtering unicode objects
             # until we fix the parsing
